function logDat = SleepLogREM(hsdfFile, varargin)

% This identifies SWS and sleep spindle episodes using ECoG (eeg) and EMG
% signals recorded in an hsdf file (i.e., low-pass filtererd and downsampled
% version of hsd file). Based on Robert Schmidt's "return_Spindle_episodes"
% used for Nico's Arky paper.

% This also identifies HVS episodes

scrsz = get(0,'ScreenSize');
scrWidth = scrsz(3);
scrHeight = scrsz(4);

% slow wave sleep detection parameters
kThr = 5; % identifes SWS in "k" factor from ECoG (see below)
minSwsDur = 30; % minimum duration of SWS episodes, in seconds

% high voltate spindle detection parameters
hOnThr = 8; % threshold for detecting HVS onset
hOffThr = 0.5; % threshold for detecting end of HVS bout
hPeakThr = 3;
hMinPeakCount = 2;
hFilt = 5;

% criteria for detecting disconnection events (rat unplugged)
disconnectLevel = 0.05; % (eeg power)
disconnectDur = 5; % seconds 

plotK = true;
fRes = 0.5;
cRange = [0.35 0.9999];
fRange = [0 100];
tRange = [0 Inf];
chan = 1;
swsIn = [];
wakeIn = [];

for i = 1:2:length(varargin)
    switch lower(varargin{i})
        case 'kthr'
            kThr = varargin{i+1};
        case 'hthr'
            hOnThr = varargin{i+1};
        case 'minswsdur'
            minSwsDur = varargin{i+1};
        case 'valideegcriterion'
            validEegCriterion = varargin{i+1};
        case 'disconnectlevel'
            disconnectLevel = varargin{i+1};
        case 'disconnectdur'
            disconnectDur = varargin{i+1};
        case 'plotk'
            plotK = varargin{i+1};
        case 'trange'
            tRange = varargin{i+1};
        case'fres'
            fRes = varargin{i+1};
        case 'sws'
            swsIn = varargin{i+1};
        case 'wake'
            wakeIn = varargin{i+1};
    end
end

% load hsdf header info for sampling rate
hsdfHeader  = getHsdHeader_ver4( hsdfFile );
Fs         = hsdfHeader.sampling_rate;
dataOffset = hsdfHeader.dataOffset;
nChan   = hsdfHeader.num_channels;   % total number of wires in the .hsd file
hsdfInfo    = dir(hsdfFile);
datalength = floor((hsdfInfo.bytes - dataOffset) / (2 * nChan));
recDur = datalength / Fs;
tRange(1) = max(tRange(1), 0);
tRange(2) = min(tRange(2), recDur);

% analyze ECoG to identify potential SWS and sleep spindle episodes
    
% load ECoG data
eeg = LoadSingleChannel(hsdfFile, chan, 'timerange', tRange)';
    
% calculate ECoG spectrogram in overlapping 1-second chunks [currently using old
% function "specgram" rather than newer "spectrogram"]
% alt: round(Fs/fRes)
[b, f, t] = specgram(eeg, round(Fs/fRes), Fs); % (signal, # pts per chunk, sampling rate)
t = t + tRange(1);

% avg power above 5 Hz is used to meansure disconnection times;
disconnectSignal = mean(abs(b(f>5,:)),1);
disconnectLocs = find(disconnectSignal < disconnectLevel);
if isempty(disconnectLocs)
    unplug_times = zeros(0,2);
else
    endLocs = disconnectLocs(find(diff(disconnectLocs) > 1));
    startLocs = disconnectLocs(find(diff(disconnectLocs) > 1) + 1);
    endLocs = [endLocs disconnectLocs(end)];
    startLocs = [disconnectLocs(1) startLocs];
    dur = endLocs - startLocs;
    startLocs = startLocs(dur > 2*disconnectDur);
    endLocs = endLocs(dur > 2*disconnectDur);
    unplug_times = [t(startLocs) t(endLocs)];
end
    
% extract power in frequency bands used to detect SWS
r1 = mean(abs(b(((f > 0.5) & (f < 8)),:))); % r1: 0.5 - 8 Hz
r2 = mean(abs(b(((f > 20) & (f < 60)),:))); % r2: 20 - 60 Hz
r3 = mean(abs(b(((f > 0.5) & (f < 4)),:))); % r3: 0.5 - 4 Hz
r4 = mean(abs(b(((f > 8) & (f < 12)),:)));  % r4: 8 - 12 Hz
    
% compute k, the factor used to identify SWS episodes
k = r1./r2 - r3./r4;
k(isnan(k)) = 0;
k = filtfilt(ones(1,round(20*fRes))/round(20*fRes), 1, k); % apply low-pass filter to k
    
% use k to identify potential SWS episodes
sws = k > kThr; % flag segments above the SWS threshold
deltaSws = diff(sws); % identify transitions between non-SWS and SWS
startLoc = find(deltaSws == 1)+1;
if sws(1)
    startLoc = [1 startLoc];
end
endLoc = find(deltaSws == -1);
if sws(end)
    endLoc = [endLoc length(t)];
end

sws_times = [t(startLoc) t(endLoc)];
if isempty(sws_times)
    dur = 0;
else
    dur = sws_times(:,2) - sws_times(:,1);
end
keepLoc = find(dur > minSwsDur);
sws_times = sws_times(keepLoc, :);
    
% compute h, the factor used to identify HVS episodes
bHvs = abs(b(f > 0.5 & f < 50,:)).^2 ./ r1.^2; % normalized spectrum for detecting HVS harmonics
h = zeros(size(r1));
for i = 1:length(h)
    pk = find(FindPeaks(bHvs(:,i), 9, 1));
    pk = pk(bHvs(pk,i) > hPeakThr);
    if length(pk) < hMinPeakCount
        pk = [];
    end
    h(i) = sum(bHvs(pk,i));
end
h = filtfilt(ones(1,round(hFilt*fRes))/round(hFilt*fRes), 1, h); % apply low-pass filter to h
    
% use h to identify potential HVS episodes
hvsOn = h > hOnThr; % flag segments above the HVS onset threshold
hvsOff = h < hOffThr; % flag segments below the HVS offset threshold
    
hvsOn = diff(hvsOn); % identify HVS onset locations
hvsOn = find(hvsOn == 1);
if h(1) > hOnThr
    hvsOn = [1 hvsOn];
end
hvsOff = diff(hvsOff); % identify transitions between non-SWS and SWS
hvsOff = find(hvsOff == 1)+1;

hvs_times = [];
i=1;
while i <= length(hvsOn)
    tmpOff = find(hvsOff > hvsOn(i));
    if isempty(tmpOff)
        hvs_times  = [hvs_times; t(hvsOn(i)) t(end)];
        hvsOn(i:end) = [];
    else
        hvs_times  = [hvs_times; t(hvsOn(i)) t(hvsOff(tmpOff(1)))];
        hvsOn(hvsOn > hvsOn(i) & hvsOn < hvsOff(tmpOff(1))) = [];
    end
    i = i + 1;
end

logDat.task = '';
logDat.rawSws = sws_times;
logDat.hvs = hvs_times;
swsDur = arrayfun(@(i) diff(sws_times(i,:)), 1:size(sws_times,1));
[~, iMax] = max(swsDur);
logDat.sws = sws_times(iMax,:);
wake = ~sws;
for i = 1:size(hvs_times,1)
    loc1 = find(t == hvs_times(i,1));
    loc2 = find(t == hvs_times(i,2));
    wake(loc1:loc2) = false;
end
deltaWake = diff(wake);
wakeUp = find(deltaWake == 1) + 1;
if wake(1)
    wakeUp = [1 wakeUp];
end
wakeEnd = find(deltaWake == -1);
if wake(end)
    wakeEnd = [wakeEnd length(t)];
end
wake_times = [t(wakeUp) t(wakeEnd)];
wakeDur = arrayfun(@(i) diff(wake_times(i,:)), 1:size(wake_times,1));
[~, iMax] = max(wakeDur);
logDat.wake = wake_times(iMax,:);
    
% make plot
if plotK
    fLoc = f >= fRange(1) & f <= fRange(2);
    f = f(fLoc);
    p = log(abs(b(fLoc,:)).^2);
    pTitle = strsplit(hsdfFile, '.');
    pTitle = strsplit(pTitle{1}, filesep);
    pTitle = [pTitle{end-1} ' - ' pTitle{end}];
    fig = figure('Position', [scrWidth/3 scrHeight/2 500 * 4 420 * 2], 'Name', pTitle, 'NumberTitle', 'off');
    subplot('Position', [0.03 0.41 0.96 0.56])
    hold on
    cR = [quantile(reshape(p, 1, []), cRange(1)) quantile(reshape(p, 1, []), cRange(2))];
    imagesc(t, f, p, cR)
    set(gca,'Yscale','lin','Ydir','normal')
    ylabel('frequency (Hz)')
    colormap(jet(200))
    fDrop = 0.015*range(f);
    if isempty(swsIn)
        for s = 1:size(sws_times,1)
            plot(sws_times(s,:), [f(end), f(end)] - fDrop, 'w', 'linewidth', 10)
            text(mean(sws_times(s,:)), f(end)+1, num2str(s), 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'center')
        end
    else
        plot(swsIn, [f(end), f(end)] - fDrop, 'w', 'linewidth', 10);
    end
    if isempty(wakeIn)
        for s = 1:size(hvs_times,1)
            plot(hvs_times(s,:), [f(end), f(end)] - fDrop, 'r', 'linewidth', 10)
        end
    else
        plot(wakeIn, [f(end), f(end)] - fDrop, 'g', 'linewidth', 10)
    end
    xlim([t(1) t(end)])
    ylim([f(1) f(end)])
    set(gca, 'XTickLabels', [])
 
    subplot('Position', [0.03 0.23 0.96 0.17])
    hold on
    set(gca, 'Box', 'on')
    plot(t,k, 'k', 'linewidth', 2)
    yR = ylim;
    if isempty(swsIn)
        plot([t(1) t(end)], [kThr kThr], '--r', 'linewidth', 1.5)   
        for s = 1:size(sws_times,1)
            plot([sws_times(s,1) sws_times(s,1)], yR, 'b', 'linewidth', 1)
            plot([sws_times(s,2) sws_times(s,2)], yR, 'b', 'linewidth', 1)
            y1 = yR(1) + 0.01*range(yR);
            y2 = yR(2) - 0.01*range(yR);
            plot(sws_times(s,:), [y1 y1], 'b', 'linewidth', 4)
            plot(sws_times(s,:), [y2 y2], 'b', 'linewidth', 4)
        end
    end
    xlabel(['time (sec)'])
    ylabel('k')
    xlim([t(1) t(end)])
    ylim(yR);
    set(gca, 'XTickLabels', [])
    
    subplot('Position', [0.03 0.05 0.96 0.17])
    hold on
    set(gca, 'Box', 'on')
    plot(t,h, 'k', 'linewidth', 2)
    yR = ylim;
    if isempty(wakeIn)
        plot([t(1) t(end)], [hOnThr hOnThr], '--m', 'linewidth', 1.5)
        plot([t(1) t(end)], [hOffThr hOffThr], '--c', 'linewidth', 1.5)
        for s = 1:size(hvs_times,1)
            plot([hvs_times(s,1) hvs_times(s,1)], yR, 'r', 'linewidth', 1)
            plot([hvs_times(s,2) hvs_times(s,2)], yR, 'r', 'linewidth', 1)
            y1 = yR(1) + 0.01*range(yR);
            y2 = yR(2) - 0.01*range(yR);
            plot(hvs_times(s,:), [y1 y1], 'r', 'linewidth', 4)
            plot(hvs_times(s,:), [y2 y2], 'r', 'linewidth', 4)
        end
    end
    xlabel(['time (sec)'])
    ylabel('h')
    xlim([t(1) t(end)])
    ylim(yR)
end
    